import 'package:flutter/material.dart';

import 'SliderContentModel.dart';
import 'SliderWidget.dart';
import 'indicatorMeasurementModel.dart';
import 'indicatorWidget.dart';

class CarouselWidgetSlider extends StatefulWidget {
  static const IndicatorModel _indicatorModel =
      IndicatorModel.animation(width: 10, height: 10, spaceBetween: 3.0);

  final List<SliderContentModel>? sliderChildren;

  final bool animation;

  final bool indicatorBottom;

  final bool showIndicator;

  final double height;

  final double width;

  final int initialPage;

  final double viewportFraction;

  final double borderRadius;

  final EdgeInsetsGeometry? margin;

  final Color? activeColor;

  final Color? disableColor;

  final IndicatorModel customizedIndicators;

  final ValueChanged<int>? onPageChanged;

  final Function(String id)? onTap;

  final double spaceBetween;

  final PageController? pageController;

  const CarouselWidgetSlider({
    Key? key,
    this.sliderChildren,
    this.height = 150,
    this.borderRadius = 5,
    this.width = double.maxFinite,
    this.margin,
    this.indicatorBottom = true,
    this.showIndicator = true,
    this.disableColor,
    this.onTap,
    this.viewportFraction = 1.0,
    this.onPageChanged,
    this.initialPage = 0,
    this.activeColor,
    this.animation = true,
    this.customizedIndicators = _indicatorModel,
    this.spaceBetween = 0,
    this.pageController,
  }) : super(key: key);

  @override
  _CarouselWidgetSliderState createState() => _CarouselWidgetSliderState();
}

class _CarouselWidgetSliderState extends State<CarouselWidgetSlider> {
  late int _page;

  @override
  void initState() {
    _page = widget.initialPage;
    super.initState();
  }

  Color get _activeColor => widget.activeColor ?? Color(0xFF10306D);
  Color get _disableColor => widget.disableColor ?? Color(0xFFC4C4C4);

  List<dynamic> get _sliderContent => widget.sliderChildren!;

  List<Widget> get _listBanners => widget.sliderChildren!
      .map((banner) => SliderWidget(
            key: Key("Banner${banner.id}"),
            sliderContentModel: banner,
            spaceBetween: widget.spaceBetween,
            onTap: widget.onTap != null
                ? () => widget.onTap!(banner.id)
                : () => print("Double Tap Banner ${banner.id}"),
            borderRadius: widget.borderRadius,
          ))
      .toList();

  List<Widget> get rowIndicator => _sliderContent
      .asMap()
      .entries
      .map((e) => CarouselIndicatorWidget(
            key: Key("Indicator${e.key}"),
            isactive: _page == e.key,
            color: _page == e.key ? _activeColor : _disableColor,
            isanimation: widget.animation,
            sizeIndicator: widget.customizedIndicators,
          ))
      .toList();

  double get _totalHeigth => widget.indicatorBottom && widget.showIndicator
      ? widget.height + widget.customizedIndicators.heightExpanded + 15
      : widget.height;

  @override
  Widget build(BuildContext context) {
    return Container(
      height: _totalHeigth,
      width: widget.width,
      margin: widget.margin ?? EdgeInsets.symmetric(horizontal: 16.0),
      child: Stack(
        children: [
          Container(
            margin: EdgeInsets.only(bottom: 40),
            height: widget.height,
            child: PageView(
              controller: widget.pageController ??
                  PageController(
                      initialPage: widget.initialPage,
                      viewportFraction: widget.viewportFraction),
              onPageChanged: (index) => _onChangePage(index),
              children: _listBanners,
            ),
          ),
          widget.showIndicator ? _indicatorRow : SizedBox()
        ],
      ),
    );
  }

  Align get _indicatorRow => Align(
        alignment: Alignment.bottomCenter,
        child: Padding(
          padding: EdgeInsets.all(8.0),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: rowIndicator,
          ),
        ),
      );

  void _onChangePage(int index) {
    if (widget.onPageChanged != null) {
      widget.onPageChanged!(index);
    }
    setState(() => _page = index);
  }
}
